//#if I2CDEV_IMPLEMENTATION == I2CDEV_ARDUINO_WIRE
#include "Wire.h"
//#endif

#include "MPU.h"
#include "Antrieb.h"

#define SERVO_OFFSET_LINKS (-20)
#define SERVO_OFFSET_RECHTS 20

#define PHI_OFFSET (-0.02)

#define P_ANTEIL 2000.0
#define I_ANTEIL 800.0
#define D_ANTEIL 200.0

#define Z_PAUSE 0
#define Z_COUNTDOWN 1
#define Z_ANFAHREN 2
#define Z_REGELN 3
#define Z_MESSWERT_AUSGABE 4

#define PWM_ANFAHREN 200
#define DAUER_MILLIS_ANFAHREN 300
#define DAUER_MILLIS_REGELN   3000

Antrieb antrieb;

//Datenaufzeichnung:
#define ANZAHL_ARR 150

//-----------------------------
double        PHI_MESS[ANZAHL_ARR];
unsigned int  DT_MESS[ANZAHL_ARR];
int           PWM_MESS[ANZAHL_ARR];
int           INDEX_MESS = 0;
//-----------------------------


double phi,phi_mittelwert;
double integ_phi=0.0,phi_alt;
double stellgroesse;
int    stellgroesse_int;

double P,I,D;
double dt,t;
unsigned int dt_int;
unsigned long ZEIT, ZEIT_ALT;

int ZUSTAND = 0;
unsigned long t_millis_int;
int i;

void initialisieren()
{
    antrieb.links( SERVO_OFFSET_LINKS );
    antrieb.rechts( SERVO_OFFSET_RECHTS );

    P = P_ANTEIL;
    I = I_ANTEIL;
    D = D_ANTEIL;

    stellgroesse=0.0;
    integ_phi = 0.0;
    phi_alt = 0.0;
    dt=0.0;
    t=0.0;
    ZEIT=micros();
    ZEIT_ALT=ZEIT;    
    
    phi_mittelwert = 0.0;
}

void setup() 
{
    Serial.begin(9600);

    initialisiereMPU();    
    antrieb.stop();
    initialisiereLEDs();
    initialisieren();
    
    ZUSTAND = 0;
}

void loop() 
{
    while(!mpuInterrupt && fifoCount < packetSize)   // wait for MPU interrupt or extra packet(s) available
    {
         DDRB|=0;
    }
//    {   
        phi = ypr[1]+PHI_OFFSET;

        phi_mittelwert = phi_mittelwert*0.99 + phi*0.01;

        stellgroesse = P*phi + I*integ_phi + D*(phi-phi_alt)/dt;

        phi_alt = phi;

        integ_phi+=phi*dt;

        ZEIT = micros();
        dt_int = ZEIT - ZEIT_ALT;
        dt = (double)dt_int;
        dt*= 0.000001;
        t+=dt;
        
        t_millis_int = (int)(t*1000.0);
        
        ZEIT_ALT = ZEIT;

        stellgroesse_int = (int)stellgroesse;

        if(stellgroesse_int>ANTRIEB_BEREICH-SERVO_OFFSET_LINKS)
            stellgroesse_int=ANTRIEB_BEREICH;
        if(stellgroesse_int>ANTRIEB_BEREICH-SERVO_OFFSET_RECHTS)
            stellgroesse_int=ANTRIEB_BEREICH;
            
        if(stellgroesse_int<-ANTRIEB_BEREICH+SERVO_OFFSET_LINKS)
            stellgroesse_int=-ANTRIEB_BEREICH;
        if(stellgroesse_int<-ANTRIEB_BEREICH+SERVO_OFFSET_RECHTS)
            stellgroesse_int=-ANTRIEB_BEREICH;

        switch(ZUSTAND)
        {
            case Z_PAUSE:
                antrieb.stop();
                rot();            
                if(phi_mittelwert>-0.4 && phi_mittelwert<0.4 && t_millis_int>2000)
                {
                    initialisieren();
                    ZUSTAND = Z_COUNTDOWN;                
                }    
                
                Serial.print("phi=");
                Serial.print(phi,DEC);
                Serial.println(" rad");
            break;
            case Z_COUNTDOWN:
                antrieb.stop();
                if((t_millis_int/200)%2==0)
                    gelb();
                else
                    aus();
                if(t_millis_int>2000)
                {
                    initialisieren();
                    antrieb.start();
                    ZUSTAND = Z_ANFAHREN;
                }
                
                if(phi_mittelwert<-0.6 || phi_mittelwert>0.6)
                {
                    initialisieren();
                    ZUSTAND = Z_PAUSE;                
                }                    
            break;
            case Z_ANFAHREN:
                gelb();
                if(phi<0.0)
                {
                    antrieb.links(-PWM_ANFAHREN+SERVO_OFFSET_LINKS);
                    antrieb.rechts(-PWM_ANFAHREN+SERVO_OFFSET_RECHTS);      
                }    
                else                                        
                {
                    antrieb.links(PWM_ANFAHREN+SERVO_OFFSET_LINKS);
                    antrieb.rechts(PWM_ANFAHREN+SERVO_OFFSET_RECHTS);      
                }
                
                if(t_millis_int>DAUER_MILLIS_ANFAHREN)
                {
                    initialisieren();
                    INDEX_MESS=0;
                    ZUSTAND = Z_REGELN;
                }
                
                if(phi_mittelwert<-0.6 || phi_mittelwert>0.6)
                {
                    initialisieren();
                    ZUSTAND = Z_PAUSE;                
                }                    
            break;
            case Z_REGELN:
                gruen();
                antrieb.links(stellgroesse_int+SERVO_OFFSET_LINKS);
                antrieb.rechts(stellgroesse_int+SERVO_OFFSET_RECHTS);      

                //Nach kurzer Einschwingzeit mit der Aufzeichnung beginnen:
//                if(t_millis_int>30 && INDEX_MESS<ANZAHL_ARR)
                if(INDEX_MESS<ANZAHL_ARR)
                {
                    PHI_MESS[INDEX_MESS] = phi;
                    DT_MESS[INDEX_MESS]  = dt_int;
                    PWM_MESS[INDEX_MESS] = stellgroesse_int;
                    INDEX_MESS++;
                }

                if(t_millis_int>DAUER_MILLIS_REGELN)
                {
                    initialisieren();
                    Serial1.begin(9600);
                    ZUSTAND = Z_MESSWERT_AUSGABE;                
                }                    
                if(phi_mittelwert<-0.6 || phi_mittelwert>0.6)
                {
                    initialisieren();
                    ZUSTAND = Z_PAUSE;                
                }                    
            break;
            case Z_MESSWERT_AUSGABE:
                 antrieb.stop();
                 gelb();
                 
                 Serial1.println(" ");
                 Serial1.println("dt/us pwm phi/rad");
                 Serial1.println(" ");
                 for(i=0;i<INDEX_MESS;i++)
                 {
                       Serial1.print(DT_MESS[i]);
                       Serial1.print(" ");
                       Serial1.print(PWM_MESS[i]);
                       Serial1.print(" ");
                       Serial1.print(PHI_MESS[i],DEC);
                       Serial1.println(" ");
                 }
                 delay(1000);
                 
                if(phi_mittelwert<-0.6 || phi_mittelwert>0.6)
                {
                    Serial1.end();
                    initialisieren();
                    ZUSTAND = Z_PAUSE;                
                }                    
            break;
        }

//    }

    checkMPUstatus();
}
